/* disk.h
   ------

   FCFS Disk abstraction.

   $Id: disk.h,v 1.7 2003/10/20 07:18:11 stewart Exp $

   (C)2003 Stewart Smith
   Distributed under the GNU Public License
*/

#ifndef __DISK_H__
#define __DISK_H__

#include "testkit/block_dev.h"
#include "testkit/types.h"

struct fcfs_disk {
  sector_t blocksnr;		/* number of blocks */
  u32 bsize;			/* Size of blocks */
  struct fcfs_disk_block *sb_block;
  struct fcfs_sb* sb;		/* The Disk SuperBlock */
  void* os_private;		/* Used by OS for it's structure */
};

struct fcfs_disk_block {
  sector_t blocknr;		/* Number of block*/
  u32   bsize;			/* Size of block*/
  char* data;				/* Block data */
  struct fcfs_disk *disk;	/* The disk the block is from */
  void* os_private;		/* The OS data structure */
};

#define BR_SECTOR_T(disk,br) (((disk)->sb->ag_blocksnr * (br)->allocation_group) + (br)->start)

struct fcfs_disk* disk_new(struct block_device *bdev);
struct fcfs_disk* disk_free(struct fcfs_disk* disk);

struct fcfs_disk_block* disk_newblock(struct fcfs_disk *disk,sector_t block);
struct fcfs_disk_block* disk_getblock(struct fcfs_disk *disk,sector_t block);
//static inline struct fcfs_disk_block* disk_freeblock(struct fcfs_disk_block* block);
//static inline struct fcfs_disk_block* disk_writeblock(struct fcfs_disk_block* block);

/* When finished using a disk block */
static inline struct fcfs_disk_block* disk_freeblock(struct fcfs_disk_block* block)
{
  ((struct buffer_head*)(block->os_private))->b_count--;
  return block;
}

/* Write a disk block immediately. */
static inline struct fcfs_disk_block* disk_writeblock(struct fcfs_disk_block* block)
{
  set_buffer_dirty((struct buffer_head*)block->os_private);
  submit_bh(WRITE,(struct buffer_head*)block->os_private);
  return block;
}


#endif
